/* 
 * $Id: inetconf.c 5272 2007-07-05 19:36:32Z ajc $
 */
/**
 * \defgroup InetCfg Functions which handle Internet domain configuration etc.
 * \ingroup CitadelConfig
 */
/*@{*/
#include "webcit.h"


/**
 * \brief display the inet config dialog 
 */
void display_inetconf(void)
{
	char buf[SIZ];
	char ename[SIZ];
	char etype[SIZ];
	int i;
	int which;

	enum {
		ic_localhost,
		ic_directory,
		ic_gwdom,
		ic_smarthost,
		ic_rbl,
		ic_spamass,
		ic_masq,
		ic_max
	};
	char *ic_spec[ic_max];
	char *ic_misc;
	char *ic_keyword[ic_max];
	char *ic_boxtitle[ic_max];
	char *ic_desc[ic_max];

	/* DON'T NEVER EVER AGAIN TRANSLATE CITADEL COMMANDS! */
	ic_keyword[0] = "localhost";
	ic_keyword[1] = "directory";
	ic_keyword[2] = "gatewaydomain";
	ic_keyword[3] = "smarthost";
	ic_keyword[4] = "rbl";
	ic_keyword[5] = "spamassassin";
	ic_keyword[6] = "masqdomain";
       

	ic_boxtitle[0] = _("Local host aliases");
	ic_boxtitle[1] = _("Directory domains");
	ic_boxtitle[2] = _("Gateway domains");
	ic_boxtitle[3] = _("Smart hosts");
	ic_boxtitle[4] = _("RBL hosts");
	ic_boxtitle[5] = _("SpamAssassin hosts");
	ic_boxtitle[6] = _("Masqueradable domains");

	ic_desc[0] = _("(domains for which this host receives mail)");
	ic_desc[1] = _("(domains mapped with the Global Address Book)");
	ic_desc[2] = _("(domains whose subdomains match Citadel hosts)");
	ic_desc[3] = _("(if present, forward all outbound mail to one of these hosts)");
	ic_desc[4] = _("(hosts running a Realtime Blackhole List)");
	ic_desc[5] = _("(hosts running the SpamAssassin service)");
	ic_desc[6] = _("(Domains as which users are allowed to masquerade)");

	for (i=0; i<ic_max; ++i) {
		ic_spec[i] = strdup("");
	}
	ic_misc = strdup("");

	serv_printf("CONF GETSYS|application/x-citadel-internet-config");
	serv_getln(buf, sizeof buf);
	if (buf[0] == '1') while (serv_getln(buf, sizeof buf), strcmp(buf, "000")) {

		extract_token(ename, buf, 0, '|', sizeof ename);
		extract_token(etype, buf, 1, '|', sizeof etype);
		which = (-1);
		for (i=0; i<ic_max; ++i) {
			if (!strcasecmp(etype, ic_keyword[i])) {
				which = i;
			}
		}

		if (which >= 0) {
			ic_spec[which] = realloc(ic_spec[which], strlen(ic_spec[which]) + strlen(ename) + 2);
			if (strlen(ic_spec[which]) > 0) strcat(ic_spec[which], "\n");
			strcat(ic_spec[which], ename);
		}
		else {
			ic_misc = realloc(ic_misc, strlen(ic_misc) + strlen(buf) + 2);
			if (strlen(ic_misc) > 0) strcat(ic_misc, "\n");
			strcat(ic_misc, buf);
		}

	}

	output_headers(1, 1, 2, 0, 0, 0);
	wprintf("<div id=\"banner\">\n");
	wprintf("<TABLE class=\"inetconf_banner\"><TR><TD>");
	wprintf("<SPAN CLASS=\"titlebar\">");
	wprintf(_("Internet configuration"));
	wprintf("</SPAN>\n");
	wprintf("</TD></TR></TABLE>\n");
	wprintf("</div>\n<div id=\"content\">\n");

	wprintf("<div class=\"fix_scrollbar_bug\">"
		"<table border=0 width=100%%><tr><td valign=top>\n");
	for (which=0; which<ic_max; ++which) {
		if (which == (ic_max / 2)) {
			wprintf("</TD><TD VALIGN=TOP>");
		}
		svprintf("BOXTITLE", WCS_STRING, ic_boxtitle[which]);
		do_template("beginbox");
		wprintf("<span class=\"menudesc\">");
		escputs(ic_desc[which]);
		wprintf("</span><br />");
		wprintf("<TABLE border=0 cellspacing=0 cellpadding=0 width=100%%>\n");
		if (strlen(ic_spec[which]) > 0) {
			for (i=0; i<num_tokens(ic_spec[which], '\n'); ++i) {
				wprintf("<TR><TD ALIGN=LEFT>");
				extract_token(buf, ic_spec[which], i, '\n', sizeof buf);
				escputs(buf);
				wprintf("</TD><TD ALIGN=RIGHT>"
					"<a href=\"save_inetconf?oper=delete&ename=");
				escputs(buf);
				wprintf("&etype=%s\" ", ic_keyword[which]);
				wprintf("onClick=\"return confirm('%s');\">",
					_("Delete this entry?"));
				wprintf("<font size=-1>");
				wprintf(_("(Delete)"));
				wprintf("</font></a></TD></TR>\n");
			}
		}
		wprintf("<FORM METHOD=\"POST\" action=\"save_inetconf\">\n");
		wprintf("<input type=\"hidden\" name=\"nonce\" value=\"%ld\">\n", WC->nonce);
		wprintf("<TR><TD>"
			"<INPUT TYPE=\"text\" NAME=\"ename\" MAXLENGTH=\"64\">"
			"<INPUT TYPE=\"hidden\" NAME=\"etype\" VALUE=\"%s\">", ic_keyword[which]);
		wprintf("</TD><TD ALIGN=RIGHT>"
			"<INPUT TYPE=\"submit\" NAME=\"oper\" VALUE=\"Add\">"
			"</TD></TR></TABLE></FORM>\n");
		do_template("endbox");
	}
	wprintf("</td></tr></table></div>\n");
	wDumpContent(1);

	for (i=0; i<ic_max; ++i) {
		free(ic_spec[i]);
	}
	free(ic_misc);
}


/**
 * \brief save changes to the inet config
 */
void save_inetconf(void) {
	char *buf;
	char *ename;
	char *etype;
	char *newconfig;

	buf = malloc(SIZ);
	ename = malloc(SIZ);
	etype = malloc(SIZ);
	newconfig = malloc(65536);

	strcpy(newconfig, "");
	serv_printf("CONF GETSYS|application/x-citadel-internet-config");
	serv_getln(buf, SIZ);
	if (buf[0] == '1') while (serv_getln(buf, SIZ), strcmp(buf, "000")) {
		extract_token(ename, buf, 0, '|', SIZ);
		extract_token(etype, buf, 1, '|', SIZ);
		if (strlen(buf) == 0) {
			/** skip blank lines */
		}
		else if ((!strcasecmp(ename, bstr("ename")))
		   &&   (!strcasecmp(etype, bstr("etype")))
		   &&	(!strcasecmp(bstr("oper"), "delete"))
		) {
			sprintf(WC->ImportantMessage, _("%s has been deleted."), ename);
		}
		else {
			if (strlen(newconfig) > 0) strcat(newconfig, "\n");
			strcat(newconfig, buf);
		}
	}

	serv_printf("CONF PUTSYS|application/x-citadel-internet-config");
	serv_getln(buf, SIZ);
	if (buf[0] == '4') {
		serv_puts(newconfig);
		if (!strcasecmp(bstr("oper"), "add")) {
			serv_printf("%s|%s", bstr("ename"), bstr("etype") );
			sprintf(WC->ImportantMessage, "%s added.", bstr("ename"));
		}
		serv_puts("000");
	}
	
	display_inetconf();

	free(buf);
	free(ename);
	free(etype);
	free(newconfig);
}



/*@}*/
